import argparse
from collections import OrderedDict
import itertools
import os.path
import shutil
import sys

import conda_build.config
from conda_build_all.resolved_distribution import ResolvedDistribution
from conda_build_all.builder import list_metas


def flatten_metas(meta_iterables):
    """
    Take a collection of metas, and compose/flatten/project into a single list.

    For example:

        A: pkg1, pkg2a
        B: pkg2b, pkg3

        Flattened([A, B]) => [pkg1, pkg2a, pkg3]
        Flattened([B, A]) => [pkg1, pkg2b, pkg3]

    The resulting list of metas will not be ordered in any particular way.

    """
    visited = {}
    for metas in meta_iterables:
        visited_this_depth = {}
        for meta in metas:
            if meta.name() not in visited:
                visited_this_depth.setdefault(meta.name(), []).append(meta)
        for name, metas in visited_this_depth.items():
            visited.setdefault(name, []).extend(metas)
    return itertools.chain.from_iterable(visited.values())


def main():
    parser = argparse.ArgumentParser(description='Removing duplicate recipes which are lower down the pecking order.')
    parser.add_argument('recipes_dirs', nargs="+",
                        help=("The directories containing recipes which should be 'flattened'."))
    parser.add_argument('--output-dir', help='Directory which should be created containing flattened recipes.',
                        default='flattened_recipes')
    args = parser.parse_args()

    meta_collections = OrderedDict([(recipes_dir, list_metas(recipes_dir))
                                    for recipes_dir in args.recipes_dirs])

    flattened = list(flatten_metas(meta_collections.values()))

    flattened_collections = OrderedDict()
    for recipe_dir, metas in meta_collections.items():
        for meta in metas:
            if meta in flattened:
                flattened_collections.setdefault(recipe_dir, []).append(meta)
 
    for recipe_dir, metas in flattened_collections.items():
        recipes_parent_dir = os.path.dirname(os.path.abspath(recipe_dir))
        for meta in metas:
            # Figure out where the recipe is, relative to the recipe dir
            meta_dir = os.path.relpath(os.path.abspath(meta.path), recipes_parent_dir)
            target_locn = os.path.join(args.output_dir, meta_dir)
            shutil.copytree(meta.path, target_locn)
            print('Copying {}'.format(meta_dir))
       

if __name__ == '__main__':
    main()



import os
import git
import shutil

import yaml


def clean_clone(repo_uri, target, remote_ref='master'):
    if os.path.exists(target):
        repo = git.Repo(target)
        remote_names = [remote.name for remote in repo.remotes]
        if 'origin' in remote_names and repo.remotes.origin.url != repo_uri:
            repo.delete_remote('origin')
            repo.create_remote('origin', repo_uri)
        elif 'origin' not in remote_names:
            repo.create_remote('origin', repo_uri)
        repo.remotes.origin.fetch()
    else:
        repo = git.Repo.clone_from(repo_uri, target)

    # Point at <remote_ref>, and clean the repo in case it has been modified.
    if remote_ref in repo.remotes.origin.refs:
        repo.head.reference = repo.remotes.origin.refs[remote_ref]
    else:
        repo.git.checkout(remote_ref)
    repo.head.reset(index=True, working_tree=True)
    repo.git.clean('-xdf')
    # Ensure we clone all submodules too.
    repo.git.submodule('update', '--init')
    return repo


def fetch_recipes(repo_defns, target):
    repo_directories = []
    for repo in repo_defns:
        if 'git_url' in repo:
            url = repo['git_url']
            ref = repo.get('git_ref', 'origin/master')
            repo_dir = os.path.join(target, repo_name(url, ref))
            repo_directories.append(clean_clone(url, repo_dir, ref).working_dir)
        else:
            raise ValueError('The repo definition {} is not '
                             'supported.'.format(repo))

    # Clear out any files which aren't part of repo_defns.
    for fname in os.listdir(target):
        fpath = os.path.abspath(os.path.join(target, fname))
        if fpath not in repo_directories:
            print('Removing {}'.format(fpath))
            shutil.rmtree(fpath)


def repo_name(repo_url, repo_ref='master'):
    # Compute the name of a directory for a git repo.
    name = repo_url
    ref = repo_ref
    replacement_chars = [':', '/', '\\', '@']
    for char in replacement_chars:
        name = name.replace(char, '-')
        ref = ref.replace(char, '-')

    return name + '__' + ref


def main():
    import argparse
    parser = argparse.ArgumentParser(description='Fetch conda recipes from git.')
    parser.add_argument('--recipes-directory', default='recipes',
                        help='The directory to put the recipes into.')
    parser.add_argument('recipes_sources',
                        help=("The source.yaml file describing the recipe "
                              "sources."))

    args = parser.parse_args()
    with open(args.recipes_sources, 'r') as fh:
        repo_definitions = yaml.safe_load(fh)
    fetch_recipes(repo_definitions, args.recipes_directory)


if __name__ == '__main__':
    main()



# TODO: Bring as much of this as possible into conda_build.source

import argparse
import os.path
from os.path import join, abspath, expanduser, isdir, isfile, normpath
import sys

from conda_build import external
from conda.fetch import download
import conda_build.config
from conda_build_all.resolved_distribution import ResolvedDistribution
from conda_build_all.builder import list_metas
from conda_build.utils import rm_rf, tar_xf, unzip, execute


def download_to_cache(meta, SRC_CACHE):
    ''' Download a source to the local cache. '''
    print('Source cache directory is: %s' % SRC_CACHE)
    if not isdir(SRC_CACHE):
        os.makedirs(SRC_CACHE) 

    fn = meta['fn'] 
    path = join(SRC_CACHE, fn)
    
    if isfile(path):
        print('Found source in cache: %s' % fn) 
    else:
        print('Downloading source to cache: %s' % fn)
        if not isinstance(meta['url'], list):
            meta['url'] = [meta['url']]

        for url in meta['url']:
            print("Downloading %s" % url)
            download(url, path)
            break
        else: # no break
            raise RuntimeError("Could not download %s" % fn)


def git_source(meta, recipe_dir, GIT_CACHE):
    if not isdir(GIT_CACHE):
        os.makedirs(GIT_CACHE)

    git = external.find_executable('git')
    if not git:
        sys.exit("Error: git is not installed")
    git_url = meta['git_url']
    if git_url.startswith('.'):
        # It's a relative path from the conda recipe
        git_url = abspath(normpath(os.path.join(recipe_dir, git_url)))
        if sys.platform == 'win32':
            git_dn = git_url.replace(':', '_')
        else:
            git_dn = git_url[1:]
    else:
        git_dn = git_url.split('://')[-1].replace('/', os.sep)
        if git_dn.startswith(os.sep):
            git_dn = git_dn[1:]
        git_dn = git_dn.replace(':', '_')
    
    cache_repo = cache_repo_arg = join(GIT_CACHE, git_dn)

    # update (or create) the cache repo
    print('Fetch {}'.format(git_url))
    if isdir(cache_repo):
        execute([git, 'fetch'], cwd=cache_repo, check_exit_code=True)
    else:
        execute([git, 'clone', '--mirror', git_url, cache_repo_arg],
                cwd=recipe_dir, check_exit_code=True)
        assert isdir(cache_repo)


def svn_source(meta, SVN_CACHE):
    ''' Download a source from SVN repo. '''
    def parse_bool(s):
        return str(s).lower().strip() in ('yes', 'true', '1', 'on')

    svn = external.find_executable('svn')
    if not svn:
        sys.exit("Error: svn is not installed")
    svn_url = meta['svn_url']
    svn_revision = meta.get('svn_rev') or 'head'
    svn_ignore_externals = parse_bool(meta.get('svn_ignore_externals') or 'no')
    if not isdir(SVN_CACHE):
        os.makedirs(SVN_CACHE)
    svn_dn = svn_url.split(':', 1)[-1].replace('/', '_').replace(':', '_')
    cache_repo = join(SVN_CACHE, svn_dn)
    if svn_ignore_externals:
        extra_args = ['--ignore-externals']
    else:
        extra_args = []
    if isdir(cache_repo):
        execute([svn, 'up', '-r', svn_revision] + extra_args,
                cwd=cache_repo, check_exit_code=True)
    else:
        execute([svn, 'co', '-r', svn_revision] + extra_args +
                [svn_url, cache_repo], check_exit_code=True)
        assert isdir(cache_repo)


def fetch_to_source_cache(meta, source_cache_root):
    orig_meta = meta
    meta = meta.get_section('source')
    if 'fn' in meta:
        download_to_cache(meta, join(source_cache_root, 'src_cache'))
    elif 'git_url' in meta:
        git_source(meta, os.path.dirname(orig_meta.meta_path),
                   join(source_cache_root, 'git_cache'))
    elif 'hg_url' in meta:
        raise NotImplementedError('hg source not yet implemented.')
    elif 'svn_url' in meta:
        svn_source(meta, join(source_cache_root, 'svn_cache'))
    elif 'path' in meta:
        pass
    else: # no source
        pass


def main():
    parser = argparse.ArgumentParser(description='Fetch the source for all of the recipes in the given directory.')
    parser.add_argument('--recipes-directory', help='The directory to look for recipes.',
                        default='recipes')
    parser.add_argument('cache_directory', help='The directory to store the source cache (aka the conda build root).')
    args = parser.parse_args()

    source_cache = os.path.abspath(args.cache_directory)
    recipes_directory = os.path.join(args.recipes_directory)
    if not os.path.exists(source_cache):
        os.makedirs(source_cache)

    # We import conda_build.config, set a value, remove conda_build.config, and re-import it.
    # We do this because conda_build has import time resolution on some of the conda_build.source
    # variables (e.g. SRC_CACHE).
    import conda_build.config
    conda_build.config.config.croot = source_cache
    sys.modules.pop('conda_build.source', None)
    import conda_build.source

    visited_sources = set()
    for meta in list_metas(recipes_directory):
        fetch_to_source_cache(meta, source_cache)


if __name__ == '__main__':
    main()



import argparse
import os.path
import sys

import conda.api
import conda.resolve

import conda_build.config
from conda_build_all.resolved_distribution import ResolvedDistribution
from conda_build_all.builder import list_metas


def main():
    parser = argparse.ArgumentParser(description='Resolve the URIs of the dependnencies of a distribution.')
    parser.add_argument('specs', nargs="+", help='The distributuion(s) to resolve.')
    parser.add_argument('--output', help='The file to output.', default='packages.txt')
    parser.add_argument('--channels', '-c', help='The channels to use when looking for distributions (defaults is not added automatically).',
                        default=[], action='append')
    args = parser.parse_args()
    print(args)
    index = conda.api.get_index(channel_urls=args.channels,
                                prepend=False)
    resolver = conda.resolve.Resolve(index)
    solved_pkgs = resolver.solve(args.specs)

    with open(args.output, 'w') as fh:
        for pkg in sorted(solved_pkgs):
            fh.write('{}{}\n'.format(index[pkg]['channel'], pkg))


if __name__ == '__main__':
    main()
